//***************************************************************************************
//  MSP430 Blink the LED Demo - Software Toggle P1.0
//
//  ACLK = n/a, MCLK = SMCLK = default DCO, P1.5 = LED on EVM430-FR6043
//
//  Description: run a dummy float operation loop filling xf[TEST_SIZE], then same
//   operations but filling xm[TEST_SIZE] using IQmathLib.
//  In between blink the LED to compare the runtime with oscilloscope.
//  Optional: enable COMPARE__MATH_VS_FLOATS to compare float vs IQmathLib results!
//
//***************************************************************************************


#include <msp430.h>
#include <stdbool.h>

//#define GLOBAL_IQ                20
#include "IQmathLib.h"

#define __nop __no_operation
#define LED 0x20                            // P1.5

#define TEST_SIZE 256

// set this to 1 to test that the math results are similar to the float results
#define COMPARE__MATH_VS_FLOATS 0
#if COMPARE__MATH_VS_FLOATS != 0
static float xmf[TEST_SIZE];
#endif


static float xf[TEST_SIZE];
static void floats_test(void)
{
    float f1 = 2.103f;
    float f2 = 1.135f;
    xf[0] = 1.0f;
    for (uint16_t i = 2; i <= TEST_SIZE; i++) {
        if (xf[i - 2U] > 1.0f) {
            xf[i - 1U] = ((f1 * (float)(i + 1U)) + (f2 * (float)(i - 1U))) / xf[i - 2U];
        } else {
            xf[i - 1U] = ((f1 * (float)(i + 1U)) + (f2 * (float)(i - 1U))) * xf[i - 2U];
        }
    }
}


// IQ20: range -2048 ... +2047.999999046, resolution 0.000000954
static _iq20 xm[TEST_SIZE];
static void math_test(void)
{
    _iq20 f1 = _IQ20(2.103f);
    _iq20 f2 = _IQ20(1.135f);
    xm[0] = _IQ20(1.0f);
    for (uint16_t i = 2; i <= TEST_SIZE; i++) {
        if (xm[i - 2U] > _IQ20(1.0f)) {
            xm[i - 1U] = _IQ20div((_IQ20mpy(f1, _IQ20((float)(i + 1U))) + _IQ20mpy(f2, _IQ20((float)(i - 1U)))), xm[i - 2U]);
        } else {
            xm[i - 1U] = _IQ20mpy((_IQ20mpy(f1, _IQ20((float)(i + 1U))) + _IQ20mpy(f2, _IQ20((float)(i - 1U)))), xm[i - 2U]);
        }
    }
}


static void blink(void)
{
    for(uint16_t i = 0; i < 25000; i++) __nop();
    P1OUT |= LED;                       // Set P1.5 (Led ON)
    for(uint16_t i = 0; i < 5000; i++) __nop();
    P1OUT &= ~LED;                      // Clear P1.5 (Led Off)
    for(uint16_t i = 0; i < 25000; i++) __nop();
}


int main(void) {
    WDTCTL = WDTPW | WDTHOLD;               // Stop watchdog timer
    PM5CTL0 &= ~LOCKLPM5;                   // Disable the GPIO power-on default high-impedance mode
                                            // to activate previously configured port settings
    P1OUT |= LED;                           // Set P1.5 (Led ON)
    P1DIR |= LED;                           // Set P1.0 to output direction
    for(uint16_t i = 0; i < 5000; i++) __nop();
    P1OUT &= ~LED;                          // Clear P1.5 (Led Off)
    for(uint16_t i = 0; i < 25000; i++) __nop();
    P1OUT |= LED;                           // Set P1.5 (Led ON)
    for(uint16_t i = 0; i < 5000; i++) __nop();
    P1OUT &= ~LED;                          // Clear P1.5 (Led Off)
    for(uint16_t i = 0; i < 65000; i++) __nop();

    for(;;) {

        blink();

        P1OUT |= LED;                       // Set P1.5 (Led ON)
        floats_test();
        P1OUT &= ~LED;                      // Clear P1.5 (Led Off)

        blink();

        P1OUT |= LED;                       // Set P1.5 (Led ON)
        math_test();
        P1OUT &= ~LED;                      // Clear P1.5 (Led Off)

#if COMPARE__MATH_VS_FLOATS != 0

        blink();

        P1OUT |= LED;                       // Set P1.5 (Led ON)
        for (uint16_t i = 0; i < TEST_SIZE; i++) {
            xmf[i] = _IQ20toF(xm[i]);
        }
        P1OUT &= ~LED;                      // Clear P1.5 (Led Off)

        blink();

        bool ok = true;
        P1OUT |= LED;                       // Set P1.5 (Led ON)
        for (uint16_t i = 0; i < TEST_SIZE; i++) {
            float diff = xmf[i] - xf[i];
            if(diff < 0) diff = -diff;
            if(diff > 0.0001f) {
                ok = false;
            }
        }
        P1OUT &= ~LED;                      // Clear P1.5 (Led Off)

        if(!ok) {
            blink();
            blink();
            blink();
            blink();
        }
#endif

    }

}
